void copyarray(byte *customchar,byte PROGMEM *progmemp){
  for(byte i = 0; i < 8; i++){
    customchar[i] = pgm_read_word_near(progmemp + i);
  }
}

int availableMemory(){
  int size = 2048;
  byte *buf;
  while ((buf = (byte *) malloc(--size)) == NULL);
  free(buf);
  return size;
}

//modulus like function to cycle numbers back to within a range
int mod(int current, int bottom, int top){
  unsigned period = (top - bottom) + 1;
  while (current < bottom || current > top){
    if(current < bottom)
      current += period;
    else
      current -= period;
  }
  return current;
}

//write the fat numbers 0-9, 10 means leave blank
void write_number(byte number, byte row){
  byte towrite;
  
  if(number > 10)
    print_error(number,__LINE__);

  if(!row){
    towrite = pgm_read_byte(numUpperLeft+number);
    lcd.write(towrite);
    towrite = pgm_read_byte(numUpperCenter+number);
    lcd.write(towrite);
    towrite = pgm_read_byte(numUpperRight+number);
    lcd.write(towrite);
  }else{
    towrite = pgm_read_byte(numLowerLeft+number);
    lcd.write(towrite);
    lcd.write(pgm_read_byte(numLowerCenter+number));
    lcd.write(pgm_read_byte(numLowerRight+number));
  }
}

void print_error(int file, int line){
  strcpy_P(outstring2, error_s);
  sprintf(outstring,outstring2, time.hour, time.minute, time.second, file);
  print_top(outstring);
  strcpy_P(outstring2, line_s);
  sprintf(outstring,outstring2,line-ARDUINO_OFFSET,(line-FILE_LENGTH)-ARDUINO_OFFSET);
  print_bot(outstring);
  analogWrite(LCD_BACKLIGHT, 128);
  while(1);
}

/*since we run continuously we must accomodate for an overflow (or wrap) of time
//millis() returns a 4-byte result or a number up to 4,294,967,296 ms or 49.7 days
//regardless, if that happens in the middle of a game it will really mess things up
//this simple method subtracts HALF_32BIT if indicated by wrap_time, so time can
//run past the half way point without any wrapping troubles and then adjust current_time
//and this function concurrently so they match */
unsigned long adjusted_millis(){  //tbt - force system time to be over and just under HALF_32BIT to make sure this works

  if(wrap_time)
    return(millis() + HALF_32BIT);

  return(millis());
}

void pause(unsigned mils){
  paused = true;
  pause_end_time = adjusted_millis() + mils;
}

//print to the upper line of the LCD
void print_top(char *outstr){
  lcd.setCursor(0,0);
  lcd.print(outstr);
  strcpy_P(outstring2, space_s);
  lcd.print(outstring2);
  //strcpy_P(outstring2, lcd_line_s);
}

//print to the lower line of the LCD
void print_bot(char *outstr){
  lcd.setCursor(0,1);
  lcd.print(outstr);
  strcpy_P(outstring2, space_s);
  lcd.print(outstring2);
}

//print to the lower line of the LCD
void print_bot_c(char *outstr){
  lcd.setCursor(0,1);
  lcd.print(outstr);
  strcpy_P(outstring2, space_s);
  lcd.print(outstring2);
  lcd.setCursor(1,1);
}

//print to the upper line of the LCD
void print_top(){
  lcd.setCursor(0,0);
  strcpy_P(outstring2, space_s);
  lcd.print(outstring2);
  //strcpy_P(outstring2, lcd_line_s);
}

//print to the lower line of the LCD
void print_bot(){
  lcd.setCursor(0,1);
  strcpy_P(outstring2, space_s);
  lcd.print(outstring2);
}

//print to the lower line of the LCD
void print_bot(byte number){
  lcd.setCursor(0,1);
  lcd.print(number,DEC);
  strcpy_P(outstring2, space_s);
  lcd.print(outstring2);
}

//get persistent data from eeprom
void get_eeprom(){
  unsigned eeprom_index = 0;

  for(int i=0; i<sizeof(eeprom); i++){
    ((byte*)&eeprom)[i] = EEPROM.read(eeprom_index++);
  }

  for(int i=0; i<sizeof(time); i++){
    ((byte*)&time)[i] = EEPROM.read(eeprom_index++);
  }

  for(int i=0; i<sizeof(alarm_onetime); i++){
    ((byte*)&alarm_onetime)[i] = EEPROM.read(eeprom_index++);
  }

  for(int i=0; i<sizeof(alarm_persistent); i++){
    ((byte*)&alarm_persistent)[i] = EEPROM.read(eeprom_index++);
  }

  strcpy_P(outstring2, geteeprom_s);
  sprintf(outstring,outstring2,eeprom_index);
  print_bot(outstring);
  delay(2000);
}

//save persistent data to eeprom
void set_eeprom(){
  unsigned eeprom_index = 0;

  for(int i=0; i<sizeof(eeprom); i++){
    EEPROM.write(eeprom_index++,((byte*)&eeprom)[i]);
  }

  for(int i=0; i<sizeof(time); i++){
    EEPROM.write(eeprom_index++,((byte*)&time)[i]);
  }

  for(int i=0; i<sizeof(alarm_onetime); i++){
    EEPROM.write(eeprom_index++,((byte*)&alarm_onetime)[i]);
  }


  for(int i=0; i<sizeof(alarm_persistent); i++){
    EEPROM.write(eeprom_index++,((byte*)&alarm_persistent)[i]);
  }

  strcpy_P(outstring2, seteeprom_s);
  sprintf(outstring,outstring2,eeprom_index);
  print_bot(outstring);
  save_eeprom = false;
  delay(2000);
}

//called when an alarm is triggered, checks if there are snoozes left and enters ALARM_TURNOFF_BUZZING or ALARM_SNOOZE_BUZZING
void check_sleep(){
  if(snooze_count){
    snooze_count--;
    add_snooze_time();
    time_mode = ALARM_SNOOZE_BUZZING;
  }else{  //if(snooze_count)
    time_mode = ALARM_TURNOFF_BUZZING;
  }

  do{
    operand1 = random(11,100);
  }while(!(operand1%10));  //if it's divisable by 10 try again, too easy

  do{
    operand2 = random(11,100);
  }while(!(operand2%10));  //if it's divisable by 10 try again, too easy

  result = operand1 * operand2;

  trigger_alarm = false;
}

void add_snooze_time(){
  alarm_onetime.hour = time.hour;
  alarm_onetime.minute = time.minute + eeprom.snooze_time_min;
  snoooze_remainder += eeprom.snooze_time_sec;
  if(snoooze_remainder >= 60){
    alarm_onetime.minute++;
    snoooze_remainder -= 60;
  }
  alarm_onetime.enabled = true;
  if(alarm_onetime.minute >= 60){
    alarm_onetime.minute = 0;
    alarm_onetime.hour++;
    if(alarm_onetime.hour >= 24){
      alarm_onetime.hour = 0;
    }
  }
}

void factory_init(){
  eeprom.magic_number = MAGIC_NUMBER;

  strcpy_P(outstring, fact_s);
  print_top(outstring);
  strcpy_P(outstring, init_s);
  print_bot(outstring);
  pause(3000);

  //set the initial time and date, this might as well be as close as possible to the time and date when the device is programmed (set these values above)
  time.ms = 0;
  time.second = START_SECONDS;
  time.minute = START_MINUTE;
  time.hour = START_HOUR;
  time.day = START_DAY;

  alarm_onetime.hour = ALARM_ONETIME_HOUR;
  alarm_onetime.minute = ALARM_ONETIME_MINUTE;
  alarm_onetime.enabled = ALARM_ONETIME_ENABLED;

  alarm_persistent.hour = ALARM_PERSISTENT_HOUR;
  alarm_persistent.minute = ALARM_PERSISTENT_MINUTE;
  alarm_persistent.enabled = ALARM_PERSISTENT_ENABLED;

  eeprom.lcd_backlight = LCD_BACKLIGHT_DEFAULT;

  eeprom.thicker = true;
  eeprom.two_six = true;

  eeprom.snooze_number = SNOOZE_NUMBER_DEF;
  eeprom.snooze_time_min = SNOOZE_TIME_DEF;
  eeprom.snooze_time_sec = SNOOZE_TIME_SEC_DEF;
  eeprom.tetris_clear_lines = AWAKE_LINES;
  
  eeprom.extras = true;
  eeprom.snooze_relative = false;

  set_eeprom();
}

//return true if the piece fits
boolean piece_fit(){

  get_adjusted_piece();

  if(!(adj_piece[0]&board[piece_y+3]) && !(adj_piece[1]&board[piece_y+2]) && !(adj_piece[2]&board[piece_y+1]) && !(adj_piece[3]&board[piece_y+0]))
    return true;
  return false;

}

void remove_piece(){

  get_adjusted_piece();

  board[piece_y+3] &= ~adj_piece[0];
  board[piece_y+2] &= ~adj_piece[1];
  board[piece_y+1] &= ~adj_piece[2];
  board[piece_y+0] &= ~adj_piece[3];
}

void set_piece(){

  get_adjusted_piece();

  board[piece_y+3] |= adj_piece[0];
  board[piece_y+2] |= adj_piece[1];
  board[piece_y+1] |= adj_piece[2];
  board[piece_y+0] |= adj_piece[3];
}

/*if this function seems to be slowing things down and we have plenty of RAM space we can increase
//speed by storing the piece definitions in four unsigned variables and eliminating the first half
//of this function */
void get_adjusted_piece(){
  adj_piece[0] = (pieces[piece_id][piece_rot] & 0xF000) >> 12;
  adj_piece[1] = (pieces[piece_id][piece_rot] & 0x0F00) >> 8;
  adj_piece[2] = (pieces[piece_id][piece_rot] & 0x00F0) >> 4;
  adj_piece[3] = (pieces[piece_id][piece_rot] & 0x000F) >> 0;
  if(piece_x <= 12){  //not sure if you can shift a negative amount
    adj_piece[0] <<= (12-piece_x);  //a for loop has enough overhead
    adj_piece[1] <<= (12-piece_x);  //that this might be less code
    adj_piece[2] <<= (12-piece_x);  //and it's definately faster (no
    adj_piece[3] <<= (12-piece_x);  //compare each time)
  }else{
    adj_piece[0] >>= (piece_x-12);  //a for loop has enough overhead
    adj_piece[1] >>= (piece_x-12);  //that this might be less code
    adj_piece[2] >>= (piece_x-12);  //and it's definately faster (no
    adj_piece[3] >>= (piece_x-12);  //compare each time)
  }
}

void get_new_piece(){

  piece_id = millis()%7;
  piece_rot = 0;

  piece_x = 7;
  if(piece_id == PIECE_I)
    piece_x--;
  piece_y = 36;
}

void zero_outs(void *ptr, unsigned memsize){
  memset(ptr,0,memsize);
}

void zero_outvs(volatile void *ptr, unsigned memsize){
  for(unsigned i = 0; i<memsize; i++){
    ((byte *)ptr)[i]=0;
  }
}

void init_board(){
  unsigned mask0 = 0xA005;
  unsigned mask1 = 0x6006;

  if(eeprom.thicker){

    board_mask = 0xF00F;

    for(byte i=1; i<MAX_BOARD_HEIGHT; i++)
      board[i] = board_mask;

  }else{

    board_mask = 0xE007;

    for(byte i=1; i<MAX_BOARD_HEIGHT; i++){
      if(i & 1)
        board[i] = mask1;
      else
        board[i] = mask0;
    }

  }

  board[0] = 0xFFFF;

}

void set_time_characters(){
  byte customchar[8];

  copyarray(customchar,upperlinechar);
  lcd.createChar(UPPER,customchar);
  copyarray(customchar,lowerlinechar);
  lcd.createChar(LOWER,customchar);
  copyarray(customchar,bothlinechar);
  lcd.createChar(BOTH ,customchar);
  copyarray(customchar,largecolonleftchar);
  lcd.createChar(LEFT_DOT,customchar);
  copyarray(customchar,largecolonrightchar);
  lcd.createChar(RIGHT_DOT,customchar);
}

//check if two left and right buttons are hit simultaneously (within TETRIS_DOUBLE_BUTTON_TIME)
void check_double(byte button){
  if(adjusted_millis()-last_tetris_button < TETRIS_DOUBLE_BUTTON_TIME && previous_button != button){
    if(eeprom.thicker == eeprom.two_six){  //this will cycle through the four states of thicker and two_six (00 01 10 11)
      eeprom.two_six ^= 1;
    }else{
      eeprom.thicker ^= 1;
    }
    save_eeprom = true;
    wait_for_new_game = true;
  }
  previous_button = button;
  last_tetris_button = adjusted_millis();
}  //void check_double()

void set_customs(){
  byte customchar[8];

  //am pm
  if(time.hour < 12){
    copyarray(customchar,small_am_char);
    lcd.createChar(SMALL_AMPM,customchar);
  }else{
    copyarray(customchar,small_pm_char);
    lcd.createChar(SMALL_AMPM,customchar);
  }

  //day of the week
  switch (time.day){
    case 0:
      copyarray(customchar,small_sun_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    case 1:
      copyarray(customchar,small_mon_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    case 2:
      copyarray(customchar,small_tue_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    case 3:
      copyarray(customchar,small_wed_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    case 4:
      copyarray(customchar,small_thu_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    case 5:
      copyarray(customchar,small_fri_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    case 6:
      copyarray(customchar,small_sat_char);
      lcd.createChar(DAY_OF_WEEK,customchar);
      break;
    default:
      error;
  }

  //snoozed, onetime alarm, persistant alarm
  if(time_mode == ALARM_SNOOZED){
    copyarray(customchar,small_s_char);
    lcd.createChar(SMALL_SOP,customchar);
  }else if(alarm_onetime.enabled){
    copyarray(customchar,small_o_char);
    lcd.createChar(SMALL_SOP,customchar);
  }else if(persistant_on()){
    copyarray(customchar,small_p_char);
    lcd.createChar(SMALL_SOP,customchar);
  }else{
    copyarray(customchar,blank_char);
    lcd.createChar(SMALL_SOP,customchar);
  }

}

//decide when to display persistant alarm indicator
//typically only on Sun - Thu, because the alarm will go off the next day
//but avoid confusion late Sat and Thu after midnight
boolean persistant_on(){
  unsigned combined_time, combined_alarm;
  
  if(!alarm_persistent.enabled)
    return false;

  combined_time = (time.hour << 8) | time.minute;
  combined_alarm = (alarm_persistent.hour << 8) | alarm_persistent.minute;
  
  switch(time.day){
    case SUN:
      if(combined_time >= combined_alarm)
        return true;
      else
        return false;
    case MON:
    case TUE:
    case WED:
    case THU:
      return true;
    case FRI:
      if(combined_time <= combined_alarm)
        return true;
      else
        return false;
    case SAT:
      return false;
    default:
      error;
  }
}  //boolean persistant_on()

/* Uses arrow keys to input an unsigned number
 * digits - the total number of digits to input, the number 12345 would have 5 digits
 */
byte input_byte(byte initial_value){
  byte i;
   char menu,mult;
   byte fact[3];
   byte div;
   unsigned factor;
   byte digits = 3;

   //div = 10^(digits-1)
//   div = 1;
//   for(i=0;i<digits-1;i++)
//      div *= 10;
   div = 100;

   //initialize fact[] with each digit
   for(i=0;i<digits;i++){
      fact[i] = initial_value / div;
      initial_value -= fact[i] * div;
      div/=10;
   }

   mult = 0;

   do{
      //lcd_cursor_home_bottom();
      lcd.setCursor(0,1);  //column,row; 0,0 upper left
      for(i=0;i<digits;i++)
         lcd.print(fact[i],DEC);
      strcpy_P(outstring2, space_s);
      lcd.print(outstring2);
      lcd.setCursor(mult,1);

      while(!button_pressed_count);
      button_pressed_count--;

      if(buttons[BUTTON_MENUUP]){
        buttons[BUTTON_MENUUP]--;
        if(fact[mult] >=9)
           fact[mult]=0;
        else
           fact[mult]++;
      }else if(buttons[BUTTON_MENUDOWN]){
        buttons[BUTTON_MENUDOWN]--;
        if(fact[mult] ==0)
           fact[mult]=9;
        else
           fact[mult]--;
      }else if(buttons[BUTTON_GOBACK]){
        buttons[BUTTON_GOBACK]--;
        if(mult>0)
           mult-- ;
        else{
           div = 100;
           factor = 0;
           for(i=0;i<digits;i++){
              factor += fact[i] * div;
              div/=10;
           }
           return factor;
        }
      }else if(buttons[BUTTON_SELECT]){
        buttons[BUTTON_SELECT]--;
        if(mult<(digits-1))
           mult++;
      }else{
        error;
      }
   }while(1);
}  //unsigned input_unsigned(unsigned digits)

enum button_enum getbutton(){
  while(!button_pressed_count);
  button_pressed_count--;

  if(buttons[BUTTON_MENUUP]){
    buttons[BUTTON_MENUUP]--;
    return BUTTON_MENUUP;
  }else if(buttons[BUTTON_MENUDOWN]){
    buttons[BUTTON_MENUDOWN]--;
    return BUTTON_MENUDOWN;
  }else if(buttons[BUTTON_GOBACK]){
    buttons[BUTTON_GOBACK]--;
    return BUTTON_GOBACK;
  }else if(buttons[BUTTON_SELECT]){
    buttons[BUTTON_SELECT]--;
    return BUTTON_SELECT;
  }else{
    error;
  }
}

